#ifndef __YAHOOMESSAGE_H__
#define __YAHOOMESSAGE_H__

#include "YahooPacket.H"
#include "StrList.H"
#include <time.h>

#define YH_MSG_NULL_F			0x00000000
#define YH_MSG_LOGIN_F			0x00000001
#define YH_MSG_LOGOUT_F			0x00000002
#define YH_MSG_ROOM_ENTER_F		0x00000004
#define YH_MSG_ROOM_LEAVE_F		0x00000008
#define YH_MSG_ROOMLIST_F		0x00000010
#define YH_MSG_BUDDYLIST_F		0x00000020
#define YH_MSG_INVITE_F			0x00000040
#define YH_MSG_USTAT_MAWAY_F	0x00000080
#define YH_MSG_USTAT_AAWAY_F	0x00000100
#define YH_MSG_USTAT_BACK_F		0x00000200
#define YH_MSG_GOTO_F			0x00000400
#define YH_MSG_SPEECH_F			0x00000800
#define YH_MSG_THOUGHT_F		0x00001000
#define YH_MSG_EMOTION_F		0x00002000
#define YH_MSG_PM_F				0x00004000
#define YH_MSG_DICE_F			0x00008000
#define YH_MSG_BSTAT_ONLINE_F	0x00010000
#define YH_MSG_BSTAT_OFFLINE_F	0x00020000
#define YH_MSG_BSTAT_AWAY_F		0x00020000
#define YH_MSG_BSTAT_BACK_F		0x00040000
#define YH_MSG_ADVERTISE_F		0x00080000
#define YH_MSG_ERR_F			0x10000000
#define YH_MSG_CONXLOST_F		0x20000000
#define YH_MSG_SELF_F			0x40000000
#define YH_MSG_KNOWN_F			0x0FFFFFFF

class YahooMessage {
	public:
		YahooMessage():_ptype(pkt_undefined),_pflag(YH_DFLAG),
				_mtype(YH_MSG_NULL_F) { };
		YahooMessage(const YahooPacket &pkt, time_t tme=0);
		YahooMessage(const YahooPacket &pkt, const string &user, time_t tme=0);
		YahooMessage(const YahooMessage &ym);
		~YahooMessage();
		YahooMessage &operator=(const YahooMessage &ym);
		void hasLostConnection() { _mtype|=(YH_MSG_ERR_F|YH_MSG_CONXLOST_F); }
		bool isLogin() const { return (_mtype&YH_MSG_LOGIN_F); };
		bool isLogout() const { return (_mtype&YH_MSG_LOGOUT_F); };
		bool isRoomEnter() const { return (_mtype&YH_MSG_ROOM_ENTER_F); };
		bool isRoomLeave() const { return (_mtype&YH_MSG_ROOM_LEAVE_F); };
		bool isRoomList() const { return (_mtype&YH_MSG_ROOMLIST_F); };
		bool isBuddyList() const { return (_mtype&YH_MSG_BUDDYLIST_F); };
		bool isBuddyOnline() const { return (_mtype&YH_MSG_BSTAT_ONLINE_F); };
		bool isBuddyOffline() const { return (_mtype&YH_MSG_BSTAT_OFFLINE_F); };
		bool isBuddyAway() const { return (_mtype&YH_MSG_BSTAT_AWAY_F); };
		bool isBuddyBack() const { return (_mtype&YH_MSG_BSTAT_BACK_F); };
		bool isInvite() const { return (_mtype&YH_MSG_INVITE_F); };
		bool isUserBack() const { return (_mtype&YH_MSG_USTAT_BACK_F); };
		bool isUserManualAway() const { return (_mtype&YH_MSG_USTAT_MAWAY_F); };
		bool isUserAutoAway() const { return (_mtype&YH_MSG_USTAT_AAWAY_F); };
		bool isUserAway() const { return (_mtype&
			(YH_MSG_USTAT_MAWAY_F|YH_MSG_USTAT_AAWAY_F)); };
		bool isUserStatus() const { return (_mtype&
			(YH_MSG_USTAT_MAWAY_F|YH_MSG_USTAT_AAWAY_F|YH_MSG_USTAT_BACK_F));};
		bool isSpeech() const { return (_mtype&YH_MSG_SPEECH_F); };
		bool isPM() const { return (_mtype&YH_MSG_PM_F); };
		bool isThought() const { return (_mtype&YH_MSG_THOUGHT_F); };
		bool isEmotion() const { return (_mtype&YH_MSG_EMOTION_F); };
		bool isAnySpeech() const { return (_mtype&
			(YH_MSG_SPEECH_F|YH_MSG_PM_F|YH_MSG_THOUGHT_F|YH_MSG_EMOTION_F)); };
		bool isDice() const { return (_mtype&YH_MSG_DICE_F); };
		bool isAd() const { return (_mtype&YH_MSG_ADVERTISE_F); };
		bool isSelf() const { return (_mtype&YH_MSG_SELF_F); }
		bool isUnknown() const { return (!((_mtype^YH_MSG_ERR_F)&
			(YH_MSG_KNOWN_F))); };
		bool isError() const { return (_mtype&YH_MSG_ERR_F); };
		bool isConnectionLost() const { return (_mtype&YH_MSG_CONXLOST_F); };
		string who() const { return (_who); };
		string room() const { return (_room); };
		string desc() const { return (_desc); };
		string msg() const { return (_msg); };
		string hdr() const { return (_hdr); };
		time_t time() const { return (_time); };
		StrList list() const { return (_list); }
		StrList debug() const { return (_debug); }
		void setmsg(const string &newmsg) {_msg=newmsg; };
	private:
		static char _wordlist[][32];
		static char _r_delimitor[8]; //BEM - mod
		static char _r_delimitor2[8]; //BEM - added
		static char _r_delimitor3[6]; //BEM - added
		static char _r_delimitor4[6]; //BEM - added
		static char _r_delimitor5[4]; //BEM - added
		yhPacketType _ptype;
		unsigned short _pflag;
		unsigned int _mtype;
		string _who;
		string _room;
		string _desc;
		string _msg;
		string _hdr;
		time_t _time;
		StrList _list;
		StrList _debug;

		int parse(const char *payload, int len, string *stor_a, int loc_a,
				string *stor_b=NULL, int loc_b=0, string *stor_c=NULL,
				int loc_c=0, string *stor_d=NULL, int loc_d=0);
		int assign(const YahooPacket &pkt, const string &user);
		void strip(string &txt);
};

#endif // __YAHOOMESSAGE_H__
